static char help[] = "Duplicates MatMult problem in parallel processing\n\n";
#include <petscksp.h>



extern PetscErrorCode ShortenMatrix(PetscMPIInt, PetscMPIInt, PetscInt, PetscInt, Mat *, Mat *, Vec *);


int main(int argc, char **args)
{
//Initialize variables
PetscMPIInt    rank, size;
PetscErrorCode       ierr;

PetscReal                 dof=2; //grid of x[0-1],t[0,1], degree of freedom=2 
PetscInt              h =8, t=8; //grid spacing
PetscInt                 Nx, Nt; //# of grid points
PetscInt total_size, final_size; //A is of size 180, C is of size 162

Mat A,C;
Vec   b; //right hand side
Vec   x;//exact solution
PetscViewer viewer;

//Initialize
PetscInitialize(&argc,&args,(char *)0,help);
MPI_Comm_size(PETSC_COMM_WORLD,&size); // Gets Number of Processors
MPI_Comm_rank(PETSC_COMM_WORLD,&rank); // Gets Rank (name) of Processor

Nx=h+1; Nt=t+1; 
total_size = (Nx)*(Nt+1)*dof;
final_size = Nx*(Nt)*dof;


//Create Vector
ierr = VecCreate(PETSC_COMM_WORLD, &x); CHKERRQ(ierr);
ierr = VecSetSizes(x, PETSC_DECIDE, final_size); CHKERRQ(ierr);
ierr = VecSetType(x,VECMPI); CHKERRQ(ierr);
ierr = VecDuplicate(x,&b); CHKERRQ(ierr);


ierr = VecAssemblyBegin(b); CHKERRQ(ierr);
ierr = VecAssemblyEnd(b); CHKERRQ(ierr);
ierr = VecAssemblyBegin(x); CHKERRQ(ierr);
ierr = VecAssemblyEnd(x); CHKERRQ(ierr);


//Create A Matrix
ierr = MatCreate(PETSC_COMM_WORLD, &A); CHKERRQ(ierr);
ierr = MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,total_size,total_size); CHKERRQ(ierr);
ierr = MatSetType(A,MATMPIAIJ); CHKERRQ(ierr);
ierr = MatMPIAIJSetPreallocation(A,total_size,PETSC_NULL, total_size, PETSC_NULL); CHKERRQ(ierr);
PetscViewerBinaryOpen(PETSC_COMM_WORLD,"mat1.dat",FILE_MODE_READ,&viewer);
MatLoad(A,viewer);
PetscViewerBinaryOpen(PETSC_COMM_WORLD,"x1.dat",FILE_MODE_READ,&viewer);
VecLoad(x,viewer);


//Create C submatrix consisting of rows/columns 18:179 of A
ierr=MatCreate(PETSC_COMM_WORLD, &C); CHKERRQ(ierr);
ierr=MatSetSizes(C,PETSC_DECIDE,PETSC_DECIDE,final_size,final_size); CHKERRQ(ierr);
ierr=MatSetType(C,MATMPIAIJ); CHKERRQ(ierr);
ierr=MatMPIAIJSetPreallocation(C,final_size,PETSC_NULL, final_size, PETSC_NULL); CHKERRQ(ierr);

ierr = ShortenMatrix(rank,size,final_size,Nx*2,&A,&C,&x); CHKERRQ(ierr);
ierr = MatAssemblyBegin(C, MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
ierr = MatAssemblyEnd(C, MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);

//Multiply
MatMult(C,x,b);
//Spotting Diffeence
VecView(b,PETSC_VIEWER_STDOUT_WORLD);

ierr = PetscFinalize(); CHKERRQ(ierr);

return 0;
}


PetscErrorCode ShortenMatrix(PetscMPIInt rank, PetscMPIInt procs, PetscInt final_size, PetscInt begin, Mat *A, Mat *AA, Vec *x){
PetscErrorCode ierr;
IS iscol,isrow;
PetscInt div,length,i;

//uses vector to determine size of IS because if done by hand
//run into problems with incompatible local sizes during matmultiplication
if(procs!=1){
	for(i=0;i<procs;i++){
		if(rank==i){
			VecGetLocalSize(*x,&length);
			VecGetOwnershipRange(*x,&div,NULL);
			ISCreateStride(PETSC_COMM_WORLD,length,div+begin,1,&iscol);
			ISCreateStride(PETSC_COMM_WORLD,length,div+begin,1,&isrow);
			ierr = MatGetSubMatrix(*A,isrow,iscol,MAT_INITIAL_MATRIX,AA); CHKERRQ(ierr);
		}
	}
}
else{
ISCreateStride(PETSC_COMM_WORLD,final_size,begin,1,&iscol);
ISCreateStride(PETSC_COMM_WORLD,final_size,begin,1,&isrow);
ierr = MatGetSubMatrix(*A,isrow,iscol,MAT_INITIAL_MATRIX,AA); CHKERRQ(ierr);
}

ierr = ISDestroy(&iscol); CHKERRQ(ierr);
ierr = ISDestroy(&isrow); CHKERRQ(ierr);

return 0;
}